@@ -3,23 +3,21 @@ import time
 import inspect
 import json
 import traceback
-from collections import defaultdict, OrderedDict
+from collections import defaultdict
 
 import numpy as np
 
-try:
+from . import test_functions as funcs
+from . import go_benchmark_functions as gbf
+from .common import Benchmark, is_xslow, safe_import
+from .lsq_problems import extract_lsq_problems
+
+with safe_import():
     import scipy.optimize
     from scipy.optimize.optimize import rosen, rosen_der, rosen_hess
     from scipy.optimize import (leastsq, basinhopping, differential_evolution,
                                 dual_annealing, OptimizeResult)
     from scipy.optimize._minimize import MINIMIZE_METHODS
-except ImportError:
-    pass
-
-from . import test_functions as funcs
-from . import go_benchmark_functions as gbf
-from .common import Benchmark
-from .lsq_problems import extract_lsq_problems
 
 
 class _BenchOptimizers(Benchmark):
@@ -420,19 +418,12 @@ class BenchLeastSquares(Benchmark):
             raise NotImplementedError
 
 
-try:
-    # the value of SCIPY_XSLOW is used to control how many repeats of each
-    # function
-    slow = int(os.environ.get('SCIPY_XSLOW', 0))
-except ValueError:
-    pass
-
-
-_func_names = os.environ.get('SCIPY_GLOBAL_BENCH', [])
-if _func_names:
-    if not slow:
-        slow = 100
-    _func_names = [x.strip() for x in _func_names.split(',')]
+# `export SCIPY_XSLOW=1` to enable BenchGlobal.track_all
+# `export SCIPY_GLOBAL_BENCH=AMGM,Adjiman,...` to run specific tests
+# `export SCIPY_GLOBAL_BENCH_NUMTRIALS=10` to specify n_iterations, default 100
+#
+# Note that it can take several hours to run; intermediate output
+# can be found under benchmarks/global-bench-results.json
 
 
 class BenchGlobal(Benchmark):
@@ -442,22 +433,20 @@ class BenchGlobal(Benchmark):
     """
     timeout = 300
 
-    _functions = OrderedDict([
+    _functions = dict([
         item for item in inspect.getmembers(gbf, inspect.isclass)
         if (issubclass(item[1], gbf.Benchmark) and
             item[0] not in ('Benchmark') and
             not item[0].startswith('Problem'))
     ])
 
-    if _func_names:
-        _filtered_funcs = OrderedDict()
-        for name in _func_names:
-            if name in _functions:
-                _filtered_funcs[name] = _functions.get(name)
-        _functions = _filtered_funcs
-
-    if not slow:
-        _functions = {'AMGM': None}
+    if not is_xslow():
+        _enabled_functions = []
+    elif 'SCIPY_GLOBAL_BENCH' in os.environ:
+        _enabled_functions = [x.strip() for x in
+                              os.environ['SCIPY_GLOBAL_BENCH'].split(',')]
+    else:
+        _enabled_functions = list(_functions.keys())
 
     params = [
         list(_functions.keys()),
@@ -467,25 +456,27 @@ class BenchGlobal(Benchmark):
     param_names = ["test function", "result type", "solver"]
 
     def __init__(self):
-        self.enabled = bool(slow)
-        self.numtrials = slow
+        self.enabled = is_xslow()
+        try:
+            self.numtrials = int(os.environ['SCIPY_GLOBAL_BENCH_NUMTRIALS'])
+        except (KeyError, ValueError):
+            self.numtrials = 100
 
         self.dump_fn = os.path.join(os.path.dirname(__file__), '..', 'global-bench-results.json')
         self.results = {}
 
     def setup(self, name, ret_value, solver):
-        if not self.enabled:
-            print("BenchGlobal.track_all not enabled --- export SCIPY_XSLOW=slow to enable,\n"
-                  "'slow' iterations of each benchmark will be run.\n"
-                  "Note that it can take several hours to run; intermediate output\n"
-                  "can be found under benchmarks/global-bench-results.json\n"
-                  "You can specify functions to benchmark via SCIPY_GLOBAL_BENCH=AMGM,Adjiman,...")
-            raise NotImplementedError()
+        if name not in self._enabled_functions:
+            raise NotImplementedError("skipped")
+
         # load json backing file
         with open(self.dump_fn, 'r') as f:
             self.results = json.load(f)
 
     def teardown(self, name, ret_value, solver):
+        if not self.enabled:
+            return
+
         with open(self.dump_fn, 'w') as f:
             json.dump(self.results, f, indent=2, sort_keys=True)
 
